/*defines a struct to hold cutoffs and constants for the program, and define their default values as well*/

#ifndef ANACUTOFF_H
#define ANACUTOFF_H

/*default values*/
#define DEFAULT_MARKER_ROUGHNESS_CUTOFF 0.5
#define DEFAULT_MARKER_CORRELATION_CUTOFF 0.975
#define DEFAULT_INTERPOLATED_DATA_CUTOFF 40
#define DEFAULT_INTERPOLATED_MARKER_CUTOFF 15
#define DEFAULT_CON_INTERPOLATED_DATA_CUTOFF 5
#define DEFAULT_CON_INTERPOLATED_MARKER_CUTOFF 3
#define DEFAULT_MARKERCORSEGMENTS 3
#define DEFAULT_NOUTLIER_CUTOFF 10

#define DEFAULT_LANESPACINGCHECK_FACTOR .2

#define DEFAULT_BORDER_COR_HARDCUTOFF 0.90
#define DEFAULT_BORDER_COR_SOFTCUTOFF 0.98
#define DEFAULT_BORDER_NEIGHBOUR_COR_CUTOFF 0.98
#define DEFAULT_BORDER_ROUGHNESS_CUTOFF 1.8

#define DEFAULT_ASYM_WINDOW_FACTOR 0.58
#define DEFAULT_ASYM_WHISKER_FACTOR 3.0
#define DEFAULT_CON_ASYM_SOFT_CUTOFF 3
#define DEFAULT_CON_ASYM_HARD_CUTOFF 5
#define DEFAULT_ASYM_THRESHOLD (2.0/255.0)
#define DEFAULT_ASYM_MAX_OFFSET (2.0/19.0)

#define DEFAULT_SLOPE_THRESHOLD 1
#define DEFAULT_MATCH_MAXOFFSET_FACTOR 20000.0
#define DEFAULT_MATCH_MIN_ROUGHNESS_FACTOR 50.0
#define DEFAULT_MATCH_MIN_VALID_THRESHOLD 1
#define DEFAULT_MATCH_CONTAM_CUTOFF 0.5
#define DEFAULT_NON_RECOM_THRESROUGH_FACTOR 80.0
#define DEFAULT_NON_RECOM_MAXDIFF 10

/*struct to hold all "magic numbers" for easy passing*/
struct AnaCutoff {
  /*for border determination*/
  float border_cor_hardcutoff;
  float border_cor_softcutoff;
  float border_neighbour_cor_cutoff;
  float border_roughness_cutoff;

 /*for result checks*/
  float marker_roughness_cutoff;
  float marker_correlation_cutoff;
  int interpolated_data_cutoff;
  int interpolated_marker_cutoff;
  int consec_data_cutoff;
  int consec_marker_cutoff;
  int markercorsegments;
  int noutlier_cutoff;
  float lanespacingcheck_factor;


  /*for asymmetry*/
  float asym_window_factor;
  float asym_whisker_factor;
  int consec_asym_soft_cutoff;
  int consec_asym_hard_cutoff;
  float asym_threshold;
  float asym_max_offset;

  /*for no growth detection*/
  int slope_threshold;
  /*for contam matching*/
  float match_maxoffset_factor;
  float match_min_roughness_factor;
  int match_min_valid_threshold;
  float match_contam_cutoff;
  float non_recom_thresrough_factor; /*imagewidth/factor gives the min thresrough for lane to be considered for contamination*/
  int non_recom_maxdiff;
};

#endif
