#include <stdio.h>
#include <stdlib.h>
#include "checkresult.h"
#include "position.h"
#include "border.h"
#include "peaks.h"


/*Check the number and location of interpolated lanes and markers
 *If too many or too big a consecutive group is interpolated in any zone, flag for inspection
 *Also check all the peak positions and make sure the distance between lanes are
 *similar to the generic spacing, flag for inspection otherwise
 */
void checkPeakPositions(FILE *ERROROUT_FILE, FILE *FLAGOUT_FILE, float **PeakPosition, float *Spacing, int Markerinc, struct GelInfo *GelInfo, struct AnaCutoff *AnaCutoff,int *num_interpolated_markers, int *num_interpolated_data, int *con_interpolated_markers, int *con_interpolated_data, int Verbose)
{

int zoneID, nodeID, laneID, markerID;
float tolerance;
int cur_con_interpolated_markers=0;
int cur_con_interpolated_data=0;
char tempstring[255];

 tolerance = GelInfo->LaneSpacing / TOLERANCEFACTOR;

 /*counting the amount of interpolated peaks*/
 for(zoneID = 0; zoneID< GelInfo->NumberOfZones; zoneID++)
   {
     num_interpolated_markers[zoneID] = 0;
     num_interpolated_data[zoneID] = 0;
     con_interpolated_markers[zoneID] = 0;
     con_interpolated_data[zoneID] = 0;
 
     for (laneID = 0; laneID < GelInfo->NumberOfLanes; laneID++)
       {
	 if (PeakPosition[zoneID+1][laneID]<=0)
      	   {
	     if (((laneID-GelInfo->MarkerOffset)%GelInfo->MarkerPeriod)==0) /*marker lane*/
	       {
		 num_interpolated_markers[zoneID]++;
		 cur_con_interpolated_markers++;
		 cur_con_interpolated_data++; /*marker part of this count as well*/
	       }
	     else
	       {
		 num_interpolated_data[zoneID]++;
		 cur_con_interpolated_data++;
	       }
	   }
	 else
	   {
	     if(cur_con_interpolated_data>con_interpolated_data[zoneID]) con_interpolated_data[zoneID]=cur_con_interpolated_data;
	     cur_con_interpolated_data=0; /*restart count*/
	     
	     if (((laneID-GelInfo->MarkerOffset)%GelInfo->MarkerPeriod)==0) /*marker lane*/
	       {
		 if(cur_con_interpolated_markers>con_interpolated_markers[zoneID]) con_interpolated_markers[zoneID]=cur_con_interpolated_markers;
		 cur_con_interpolated_markers=0;
	       }
	   }
       }

     
     /*set interpolated data counts to negative if the zone is marker matched only, so as to not cause failure*/
     if (fabs(Spacing[zoneID]-GelInfo->LaneSpacing) > tolerance && Markerinc == 1) 
       {
	 num_interpolated_data[zoneID]*=-1;
	 con_interpolated_data[zoneID]*=-1;
       }
     
  /*check interpolated marker and interpolated data if necessary to fail gel*/
      if(num_interpolated_data[zoneID] >= AnaCutoff->interpolated_data_cutoff)
	{
	  GelInfo->GelPassed = 0;

	  sprintf(tempstring, "Interpolated %i data lanes in node %i, Gel FAILED\n", num_interpolated_data[zoneID], zoneID+1);
	    if (Verbose) fprintf(stderr, "%s", tempstring);
	     fprintf(ERROROUT_FILE, "%s", tempstring);
	}
      if(num_interpolated_markers[zoneID]>=AnaCutoff->interpolated_marker_cutoff)
	{
	  GelInfo->GelPassed = 0;

	  sprintf(tempstring,"Interpolated %i marker lanes in node %i, Gel FAILED\n", num_interpolated_markers[zoneID], zoneID+1);
	  if (Verbose) fprintf(stderr, "%s", tempstring);
	  fprintf(ERROROUT_FILE, "%s", tempstring);
	}
      if(con_interpolated_data[zoneID] >= AnaCutoff->consec_data_cutoff)
	{
	  GelInfo->GelPassed = 0;
	  
	  sprintf(tempstring,"Interpolated %i consecutive data lanes in node %i, Gel FAILED\n", con_interpolated_data[zoneID], zoneID+1);
	  if (Verbose) fprintf(stderr, "%s", tempstring);
	  fprintf(ERROROUT_FILE, "%s", tempstring);
	}
      if(con_interpolated_markers[zoneID]>=AnaCutoff->consec_marker_cutoff)
	{
	  GelInfo->GelPassed = 0;
	  
	  sprintf(tempstring, "Interpolated %i consecutive marker lanes in node %i, Gel FAILED\n", con_interpolated_markers[zoneID], zoneID+1);
	  if (Verbose) fprintf(stderr, "%s", tempstring);
	  fprintf(ERROROUT_FILE, "%s", tempstring);
	}
   }

 /*output to file*/
 fprintf(FLAGOUT_FILE, "number of interpolated marker lanes in each zone:"); 
 for (zoneID = 0; zoneID < GelInfo->NumberOfZones; zoneID++)
   {
     fprintf(FLAGOUT_FILE, "\t%i", num_interpolated_markers[zoneID]);
   }
 fprintf(FLAGOUT_FILE, "\n");
 fprintf(FLAGOUT_FILE, "number of interpolated data lanes in each zone:"); 
 for (zoneID = 0; zoneID < GelInfo->NumberOfZones; zoneID++)
   {
     fprintf(FLAGOUT_FILE, "\t%i", num_interpolated_data[zoneID]);
   }
 fprintf(FLAGOUT_FILE, "\n");


 fprintf(FLAGOUT_FILE, "maximum number of consecutive interpolated marker lanes in each zone:"); 
 for (zoneID = 0; zoneID < GelInfo->NumberOfZones; zoneID++)
   {
     fprintf(FLAGOUT_FILE, "\t%i", con_interpolated_markers[zoneID]);
   }
 fprintf(FLAGOUT_FILE, "\n");
 fprintf(FLAGOUT_FILE, "maximum number of consecutive interpolated data lanes in each zone:"); 
 for (zoneID = 0; zoneID < GelInfo->NumberOfZones; zoneID++)
   {
     fprintf(FLAGOUT_FILE, "\t%i", con_interpolated_data[zoneID]);
   }
 fprintf(FLAGOUT_FILE, "\n");

 /*check lane spacing*/
 for (nodeID = 1; nodeID< GelInfo->NumberOfZones+1; nodeID++)
   {
     for (laneID = 0; laneID < GelInfo->NumberOfLanes-1; laneID++)
       {
	 if(fabs(fabs(PeakPosition[nodeID][laneID+1])-fabs(PeakPosition[nodeID][laneID])-GelInfo->LaneSpacing) > GelInfo->LaneSpacing*AnaCutoff->lanespacingcheck_factor)
	   {
	     GelInfo->GelPassed = 0;
	     sprintf(tempstring, "Lane Spacing between lane %i and %i in node %i is %f, Gel FAILED\n", GelInfo->NumberOfLanes-1-laneID, GelInfo->NumberOfLanes-1-(laneID+1), nodeID, fabs(PeakPosition[nodeID][laneID+1])-fabs(PeakPosition[nodeID][laneID]));
	     if (Verbose) fprintf(stderr, "%s", tempstring);
	     fprintf(ERROROUT_FILE, "%s", tempstring);
	   }
       }
   }

}

/*a piece-wise linear line that connects all the nodes for a lane is extracted for each marker lane.
 *these are representation of the middle of the marker lanes
 *these lines are then tested for roughness and correlation in segments to catch missing markers and lane skipping
 *Any low roughness cases will flag the gel for inspection
 */
void checkMarkers(FILE *ERROROUT_FILE, FILE *FLAGOUT_FILE, PIX *pixGelin, float **PeakPosition, float *NodePosition, int *ZoneID, struct GelInfo *GelInfo, struct AnaCutoff *AnaCutoff, float **MarkerCorrelation, float **MarkerRoughness, int Verbose)
{

PIX *pixtemp;
UINT32  *lineinlow, *lineinhigh, *datain, *lineout, *dataout;
INT32   wplin, wplout;
int	markerID, laneID, nodeID;
float	y, slope;
int	halfwidth, output_width;
int	xindex, xstop, ylow, yhigh;
 float   ftemp;
 int nummarker, maxoffset;
 int start, stop, segment, i;
 int startsegment, stopsegment;
 char tempstring[255];

 if(pixGetDepth(pixGelin) != 8) 
   {
     fprintf(stderr, "Writelanepix function only support 8-bit pix\n");
     exit(1);
   }

 nummarker = (GelInfo->NumberOfLanes-1-GelInfo->MarkerOffset)/GelInfo->MarkerPeriod+1;

 if ((pixtemp = pixCreate(GelInfo->ImageWidth, nummarker, 8)) == NULL) {
   fprintf(stderr, "Cannot create pix for marker comparison\n");
   exit(1);
 } 
 datain = pixGetData(pixGelin);
 wplin = pixGetWpl(pixGelin);
 dataout = pixGetData(pixtemp);
 wplout = pixGetWpl(pixtemp);

 /*take middle of all markers and store in a pix for correlation calculation*/
 for (markerID = 0; markerID < nummarker; markerID++)
   { 
     laneID = markerID*GelInfo->MarkerPeriod+GelInfo->MarkerOffset;
     lineout = dataout + markerID*wplout;

     for (nodeID = 0; nodeID < GelInfo->NumberOfZones+1; nodeID++) 
       {   
	 slope = (fabs(PeakPosition[nodeID+1][laneID])-fabs(PeakPosition[nodeID][laneID]))/(NodePosition[nodeID+1]-NodePosition[nodeID]);
	 xindex = (int)(NodePosition[nodeID]+0.5);
	 xstop = (int)(NodePosition[nodeID+1]+0.5);
	 if (nodeID == GelInfo->NumberOfZones) xstop++;
	 while (xindex < xstop) {
	   y = slope * ((float)xindex-NodePosition[nodeID])+fabs(PeakPosition[nodeID][laneID])-0.5;
	   ylow=floor(y);
	   yhigh=ceil(y);
	   lineinlow = datain + (ylow) * wplin;
	   if (ylow == yhigh) {
	     ftemp = (float) GET_DATA_BYTE(lineinlow, xindex);
	     SET_DATA_BYTE(lineout, xindex, (UINT8)ftemp);
	   } else {
	     lineinhigh = datain + (yhigh) * wplin;
	     ftemp = floor((float)GET_DATA_BYTE(lineinlow, xindex)*((float)yhigh-y) +
				     (float)GET_DATA_BYTE(lineinhigh, xindex)*(y-(float)ylow));
	     SET_DATA_BYTE(lineout, xindex, (UINT8)ftemp);
	   }
	   xindex++;
	 }
       }
   }

 /*divide into sections based on zones and calculate the correlation with next marker*/
 /*if below theshold, the peak will be refit in CorrectMarker*/
 maxoffset = GelInfo->ImageWidth / OFFSETFACTOR;
 start = GelInfo->StopPreData;
 if (start < maxoffset) start = maxoffset;
 startsegment = GelInfo->FirstDataZone;
 stopsegment = GelInfo->FirstDataZone-1;
 for (segment=0; segment<AnaCutoff->markercorsegments; segment++)
   {
     startsegment = stopsegment+1;
     for(i = start; i < GelInfo->ImageWidth-maxoffset; i++)
       {
	 if(ZoneID[i] >= (int) ((float) (segment+1)/(float) AnaCutoff->markercorsegments* ((float)(GelInfo->NumberOfZones- GelInfo->FirstDataZone)))+GelInfo->FirstDataZone||ZoneID[i]==-1) break;
       }
     stopsegment = ZoneID[i-1];
     stop = i;
     if (stop >= GelInfo->StopData-maxoffset) stop = GelInfo->StopData-maxoffset-1;
     for(markerID = 0; markerID<nummarker-1;markerID++)
       {  
	 MarkerCorrelation[nummarker - 2 - markerID][segment] = correlation(pixtemp,markerID, markerID+1, start, stop, maxoffset);
      	 MarkerRoughness[nummarker - 1 - markerID][segment] = roughness(pixtemp, markerID, start, stop);
	 if (MarkerRoughness[nummarker - 1 - markerID][segment]<AnaCutoff->marker_roughness_cutoff)
	   {
	     GelInfo->GelPassed = 0;
	     sprintf(tempstring, "Roughness of marker %i at between nodes %i and %i is %f, Gel FAILED.\n", GelInfo->NumberOfLanes-1 -(GelInfo->MarkerPeriod*(markerID)+GelInfo->MarkerOffset), startsegment+1, stopsegment+1, MarkerRoughness[nummarker - 1 - markerID][segment]);
	     if (Verbose) fprintf(stderr, "%s", tempstring);
	     fprintf(ERROROUT_FILE, "%s", tempstring);
	     
	   }
       }
     markerID = nummarker-1;	
     MarkerRoughness[nummarker - 1 - markerID][segment] = roughness(pixtemp, markerID, start, stop);
     if (MarkerRoughness[nummarker - 1 - markerID][segment]<AnaCutoff->marker_roughness_cutoff)
       {
	 GelInfo->GelPassed = 0;
	 sprintf(tempstring, "Roughness of marker %i at between nodes %i and %i is %f, Gel FAILED.\n", GelInfo->NumberOfLanes-1 -(GelInfo->MarkerPeriod*(markerID)+GelInfo->MarkerOffset), stopsegment+1, MarkerRoughness[nummarker - 1 - markerID][segment]);
	 if (Verbose) fprintf(stderr, "%s", tempstring);
	 fprintf(ERROROUT_FILE, "%s", tempstring);
       }
     start = stop;
   }

 /*output to file*/
 fprintf(FLAGOUT_FILE, "marker correlation before smoothing:\n");
 for (markerID = 0; markerID < nummarker-1; markerID++)
   {
     for (segment = 0; segment < AnaCutoff->markercorsegments; segment++)
       {
	 if(MarkerCorrelation[markerID][segment]<AnaCutoff->marker_correlation_cutoff)
	   {
	     fprintf(FLAGOUT_FILE, "\t%f", -1*MarkerCorrelation[markerID][segment]);
	   }
	 else 
	   {
	     fprintf(FLAGOUT_FILE, "\t%f", MarkerCorrelation[markerID][segment]); 
	   }
       }
     fprintf(FLAGOUT_FILE, "\n");
   }
 
 fprintf(FLAGOUT_FILE, "marker roughness before smoothing:\n");
 for (markerID = 0; markerID < nummarker; markerID++)
   {  
      for (segment = 0; segment < AnaCutoff->markercorsegments; segment++)
	{
	 if(MarkerRoughness[markerID][segment]<AnaCutoff->marker_roughness_cutoff)
	   {
	     fprintf(FLAGOUT_FILE, "\t%f", -1*MarkerRoughness[markerID][segment]);
	   }
	 else
	   {
	     fprintf(FLAGOUT_FILE, "\t%f", MarkerRoughness[markerID][segment]);
	   }
	}
      fprintf(FLAGOUT_FILE, "\n");
   }
 
 /*   pixWrite("/home/rwong/markercor.png", pixtemp, IFF_PNG); */
 pixDestroy(&pixtemp);
}

/*rematches a single node 
 *no psize support for single peak matching
*/
void matchSinglePeak(INT32 **ZoneProjection, struct GelInfo *GelInfo, float **PeakPosition, int zoneID, int laneID)
{

float	*pcentroid, *psize;
float	sigma;
int	length, loch, hich, ifwhm, maxpk, npeaks, nodeID;
int	i, j, k;
float	template, tolerance, current_tolerance, previouspeak;
int	itemp, jtemp;
float	ftemp1, ftemp2;
float   maxpsize, currentpsize;

  maxpk = GelInfo->NumberOfLanes*2;
  pcentroid = calloc(maxpk, sizeof(float));
  psize = calloc(maxpk, sizeof(float));
  length = GelInfo->ImageHeight;
  ifwhm = (int)(GelInfo->LaneSpacing * FWHM_FACTOR + 0.5);
  sigma = SIGMA_CUTOFF;
  tolerance = GelInfo->LaneSpacing / TOLERANCEFACTOR;
  
  nodeID = zoneID + 1;
 
  /*used found peaks as cutoff, finding peaks within one marker period only*/
  if (laneID - GelInfo->MarkerPeriod > 0)
    {
      loch = (int) (fabs(PeakPosition[nodeID][laneID-GelInfo->MarkerPeriod]) - fabs(GelInfo->LaneSpacing) + 0.5); /*start finding peaks a bit  before first lane*/
    }
  else 
    {
      loch = (int) (fabs(PeakPosition[nodeID][0]) - fabs(GelInfo->LaneSpacing) + 0.5); /*start finding peaks a bit  before first lane*/
    }
  if (laneID + GelInfo->MarkerPeriod < GelInfo->NumberOfLanes)
    {
      hich = (int) (fabs(PeakPosition[nodeID][laneID+GelInfo->MarkerPeriod]) + fabs(GelInfo->LaneSpacing) + 0.5); /*and after last lane*/
    }
  else 
    {
      hich = (int) (fabs(PeakPosition[nodeID][GelInfo->NumberOfLanes-1]) + fabs(GelInfo->LaneSpacing) + 0.5); /*and after last lane*/
    }
  npeaks = specFindPeak(ZoneProjection[zoneID], length, pcentroid, psize, loch, hich, ifwhm, sigma, maxpk);

  template = fabs(PeakPosition[nodeID][laneID]);
/*   fprintf(stderr, "%f\n", PeakPosition[nodeID][laneID]); */
  PeakPosition[nodeID][laneID]=0;  
  /*matching peak*/
  
  current_tolerance = tolerance;
  
  for(i = 0; i < npeaks; i++) 
    {
      if (fabs(template-pcentroid[i]) < current_tolerance) 
	{
	  current_tolerance = fabs(template-pcentroid[i]);
	  PeakPosition[nodeID][laneID] = pcentroid[i];
	  pcentroid[i] = -pcentroid[i]; /*if close enough to a lane, should not belong to any other lanes*/
	}
      if (pcentroid[i]>template+current_tolerance) 
	{
	  break; /*done when we are looking pass the peak*/
	}
    }
/*   fprintf(stderr, "%f\n", PeakPosition[nodeID][laneID]); */
  free(pcentroid);
}

/*markers nodes in segments with poor correlation (possibly noisy) are rematched using the filtered zone projection
 */
int correctMarkers(FILE *FLAGOUT_FILE, INT32 **ZoneProjection, float **PeakPosition, struct GelInfo *GelInfo, struct AnaCutoff *AnaCutoff, float **MarkerCorrelation, int Verbose)
{

  int	i, markerID, laneID, ncorrected, nummarker;
  int   segment, zoneID, nodeID;
  int   needfilling=0;
  int **needadjust;



  ncorrected = 0;
  nummarker = (GelInfo->NumberOfLanes-1-GelInfo->MarkerOffset)/GelInfo->MarkerPeriod+1;
  
  needadjust = (int **)malloc((nummarker) * sizeof(int*));
  for (markerID = 0; markerID < nummarker; markerID++) 
    {
      needadjust[markerID] =(int*) calloc(AnaCutoff->markercorsegments, sizeof(int)); /*identify peaks that needs adjusting*/
    } 

  for (segment=0; segment<AnaCutoff->markercorsegments; segment++)
    { 
      /*finding segments that needs correcting*/
      for(markerID = 0; markerID<nummarker-1;markerID++)
	{

	  if (MarkerCorrelation[nummarker - 2 - markerID][segment]<AnaCutoff->marker_correlation_cutoff) /*a pair out of tolerance*/
	    {	     

/* 	      fprintf(stderr, "found marker pair %i at segment %i has low correlation\n", markerID, segment); */

	      if (markerID==0) /*always correct edges when identified*/
		{
		  needadjust[markerID][segment] = 1;
		}
	      else if (markerID==nummarker-2)
		{
		  needadjust[markerID+1][segment] = 1;
		}
	      else if (MarkerCorrelation[nummarker - 2 - (markerID-1)][segment]<AnaCutoff->marker_correlation_cutoff) /*if correlation on both sides are bad*/
		{
		  needadjust[markerID][segment] = 1;
		}
	      else if (MarkerCorrelation[nummarker - 2 - (markerID+1)][segment]>=AnaCutoff->marker_correlation_cutoff) /*if low correlation pair is isolated*/
		{
		  needadjust[markerID+1][segment] = 1;
		  needadjust[markerID][segment] = 1;
		}
	    }
	}
      
      /*correcting peaks*/
      for(markerID = 0; markerID<nummarker;markerID++)
	{
	  laneID = markerID*GelInfo->MarkerPeriod+GelInfo->MarkerOffset;
	  if(needadjust[markerID][segment]==1) /*correct only if correlation to both sides are bad, or if at edge, or isolated */
	    { 
	      needfilling =0;
	      for(zoneID = 0; zoneID < GelInfo->NumberOfZones; zoneID++)		 
		{
		  nodeID = zoneID+1;
		  if(segment == (int) floor((float)(zoneID-GelInfo->FirstDataZone)*(float) AnaCutoff->markercorsegments / (float)(GelInfo->NumberOfZones- GelInfo->FirstDataZone))) /*zone in segment*/
		    {
		      ncorrected++;
/* 		      fprintf(stderr, "correcting segment %i marker %i zone %i\n", segment, markerID, zoneID); */
		      
		      /*correct the one peak, will be 0 if not matched*/
		      matchSinglePeak(ZoneProjection, GelInfo, PeakPosition, zoneID, laneID);
		      if (PeakPosition[nodeID][laneID]==0)needfilling=1;
		      /*all estimated peaks based on outlier has to be refilled*/
		      i = laneID - 1;
		      while (i >= 0) {
			if (PeakPosition[nodeID][i] < 0) {
			  PeakPosition[nodeID][i] = 0;
			  i--;
			  needfilling=1;
			} else {
			  break;
			}
		      }
		      i = laneID + 1;
		      while (i < GelInfo->NumberOfLanes) {
			    if (PeakPosition[nodeID][i] < 0) {
			      PeakPosition[nodeID][i] = 0;
			      i++;
			      needfilling=1;
			    } else {
			      break;
			    }
		      }
		      if (needfilling) fillEmpties(PeakPosition[nodeID], GelInfo);	 
		    }
		}
	    }
	}
    }
  
  if (Verbose) fprintf(stderr, "Corrected %d peaks for low neighbour markers correlation\n", ncorrected);
  fprintf(FLAGOUT_FILE, "Number of peaks corrected for low neighbour markers correlation:\t%i\n", ncorrected);
 

  
  for (markerID = 0; markerID < nummarker; markerID++) {
    free(needadjust[markerID]);
  } 
  free(needadjust);

  return ncorrected;
}
