/*contains function to write a PNG of the gel with 
 *node and peak position information overlaid.  
 *Interpolated peaks are shown in red for diagnostic purposes
 */

#include <stdio.h>
#include <stdlib.h>
#include "diagnostic.h"

/*output a idiagnostic image in png format with peaks and nodes position marked
 *interpolated peaks are shown in red, the rest are shown in green
 */
void outputDiagnostic (PIX *pixGel, char *fileDiagnosticImage, float **PeakPosition, float* NodePosition, int *ZoneID, struct GelInfo *GelInfo) {

UINT32	*lineR, *lineG, *lineB;
UINT32	*dataR, *dataG, *dataB;
INT32	wplR, wplG, wplB;
PIX	*pixRed, *pixGreen, *pixBlue, *pixRGB;
int	i, j, k, peak, colour;
int	*startzone, *stopzone;
char	fullName[FILE_NAME_LEN];

  sprintf(fullName,"%s%s",GelInfo->ProjectDir,fileDiagnosticImage); 

  pixRed = pixCopy(NULL, pixGel);
  pixGreen = pixCopy(NULL, pixRed);
  pixBlue =  pixCopy(NULL, pixRed);
  
  dataR = pixGetData(pixRed);
  dataG = pixGetData(pixGreen);
  dataB = pixGetData(pixBlue);
  wplR = pixGetWpl(pixRed);
  wplG = pixGetWpl(pixGreen);
  wplB = pixGetWpl(pixBlue);
  
  startzone = calloc(GelInfo->NumberOfZones, sizeof(int));
  stopzone = calloc(GelInfo->NumberOfZones, sizeof(int));
  for (i = 0; i < GelInfo->NumberOfZones; i++) startzone[i] = GelInfo->ImageWidth-1;
  /*set start and stop point for each zone, does not include things before StartPreData and after StopData*/
  for (i = 0; i < GelInfo->ImageWidth; i++) {
    if (ZoneID[i] < 0) continue;
    if (startzone[ZoneID[i]] > i) startzone[ZoneID[i]] = i;
    if (stopzone[ZoneID[i]] < i) stopzone[ZoneID[i]] = i;
  } 
  
  for (i = 0; i < GelInfo->NumberOfZones; i++) {
    for (j = 0; j < GelInfo->NumberOfLanes; j++) {
      peak = (int)(fabs(PeakPosition[i+1][j])+0.5);
      colour = 0;
      /*draw lines at peak in each zone*/
      if (PeakPosition[i+1][j] > 0) colour = 1; 
      /*green for found PeakPositions*/
      if (colour) {
        lineR = dataR + peak * wplR;
	lineG = dataG + peak * wplG;
	lineB = dataB + peak * wplB;
        for (k = startzone[i]; k <= stopzone[i]; k++) {
          SET_DATA_BYTE(lineG, k, 128);
	  SET_DATA_BYTE(lineR, k, 0);
	  SET_DATA_BYTE(lineB, k, 0);
        }
      } else { /*red for estimated position*/
        lineR = dataR + peak * wplR;
	lineG = dataG + peak * wplG;
	lineB = dataB + peak * wplB;
        for (k = startzone[i]; k <= stopzone[i]; k++) {
          SET_DATA_BYTE(lineR, k, 128);
	  SET_DATA_BYTE(lineG, k, 0);
	  SET_DATA_BYTE(lineB, k, 0);
        }
      } 
        /*draw in nodepositions*/
      for (k = peak-1; k < peak+2; k++){
	
	lineR = dataR + k * wplR;
	lineG = dataG + k * wplG;
	lineB = dataB + k * wplB;
	if (colour) {
	  SET_DATA_BYTE(lineG, NodePosition[i+1], 128);
	  SET_DATA_BYTE(lineR, NodePosition[i+1], 0);
	  SET_DATA_BYTE(lineB, NodePosition[i+1], 0);
	} else {
	  SET_DATA_BYTE(lineR, NodePosition[i+1], 128);
	  SET_DATA_BYTE(lineG, NodePosition[i+1], 0);
	  SET_DATA_BYTE(lineB, NodePosition[i+1], 0);
	}
      }
    }
    
  }
  
  pixRGB = pixCreateRGBImage(pixRed, pixGreen, pixBlue);
  pixWrite(fullName, pixRGB, IFF_PNG);
   
  pixDestroy(&pixRed);
  pixDestroy(&pixGreen);
  pixDestroy(&pixBlue);
  pixDestroy(&pixRGB);
  free(startzone);
  free(stopzone);
}

