/*holds function to read 16 bit grayscale tiff*/

#include <stdio.h>
#include <stdlib.h>
#include "readtiff.h"
#include "tiffio.h"

/*reads in a 16bit grayscale gel image (TIFF) to a PIX
 *able to support different image orientation defined using well_edge
 */
PIX *pixReadTIFF16Gray(FILE *FLAGOUT_FILE, char *filename, struct GelInfo *GelInfo,  int well_edge,  int Verbose) {

INT32		width, height, depth, rowbyte, wpl;
UINT32		*data, *line;
PIX		*pixIN;
PIXCMAP		*cmap;
TIFF		*tif;
uint32 		imagelength, imagewidth, row, col, linesize; 
tdata_t 	buf;
uint16 		*myline;
uint8		*byteline;
uint16 		bitspersample, photometric;
char		fullName[FILE_NAME_LEN];

  sprintf(fullName,"%s%s",GelInfo->ProjectDir,filename);
  tif = TIFFOpen(fullName, "r");
  if (tif) {
     if (Verbose) fprintf(stderr, "Reading image %s\n", filename);
     TIFFGetField(tif,TIFFTAG_BITSPERSAMPLE , &bitspersample);
     if (bitspersample != 16) {
       fprintf(stderr, "TIFFTAG_BITSPERSAMPLE = %d, wrong image format!\n", bitspersample);
       exit(1);
     }
     TIFFGetField(tif,TIFFTAG_PHOTOMETRIC , &photometric);
     if (photometric != PHOTOMETRIC_MINISWHITE) {
       fprintf(stderr, "TIFFTAG_PHOTOMETRIC = %d, not a grayscale image with white as minimum\n", photometric);
       exit(1);
     } 
     

     TIFFGetField(tif, TIFFTAG_IMAGELENGTH, &imagelength);
     TIFFGetField(tif, TIFFTAG_IMAGEWIDTH, &imagewidth);   

     linesize = TIFFScanlineSize(tif);
     if (linesize != 2 * imagewidth) {
       fprintf (stderr,"line size = %d bytes, which is different from 2 * width!\n", linesize);
       exit(1);
     }
     
     if (well_edge % 2 == 0) /*portrait*/
       {
	 GelInfo->ImageHeight = imagelength;
	 GelInfo->ImageWidth = imagewidth;
	 width = (INT32) imagewidth;
	 height = (INT32)imagelength;
       } else { /*landscape*/
	 GelInfo->ImageHeight = imagewidth;
	 GelInfo->ImageWidth = imagelength;
	 width = (INT32) imagelength;
	 height = (INT32)imagewidth;
       }

     if (Verbose) fprintf(stderr, "Image width, height = %d, %d pixels\n", GelInfo->ImageWidth, GelInfo->ImageHeight);
     fprintf(FLAGOUT_FILE, "Image width:\t%d\nImage height:\t%d\n", GelInfo->ImageWidth, GelInfo->ImageHeight);

     buf = _TIFFmalloc(TIFFScanlineSize(tif));
     depth = 16;
     if ((pixIN = pixCreate(width, height, depth)) == NULL) {
       fprintf(stderr, "Cannot create pix for %s\n",filename);
       exit(1);
     }
     cmap = NULL;
     wpl = pixGetWpl(pixIN);
     data = pixGetData(pixIN);
     pixSetColormap(pixIN, cmap);

     switch (well_edge)
       {
       case 0:
	 for (row = 0; row < imagelength; row++) {
	   TIFFReadScanline(tif, buf, row, 0);
	   myline = buf;
	   line = data + row * wpl;
	   byteline = buf;
	   for (rowbyte = 0; rowbyte < linesize - 1; rowbyte+=2) {
#ifdef _LITTLE_ENDIAN_
	     SET_DATA_BYTE(line, rowbyte, byteline[rowbyte+1]);
	     SET_DATA_BYTE(line, rowbyte+1, byteline[rowbyte]);
#else  /* _BIG_ENDIAN_ */
	     SET_DATA_BYTE(line, rowbyte, byteline[rowbyte]);
	     SET_DATA_BYTE(line, rowbyte+1, byteline[rowbyte+1]);
#endif
	   }
	 }   
	 break;
	 
       case 1:
	 for (col = 0; col < imagelength; col++) {
	   TIFFReadScanline(tif, buf, col, 0);
	   myline = buf;
	   byteline = buf;
	   
	   for (row = 0; row < imagewidth; row++) {
	     line = data + row * wpl;
#ifdef _LITTLE_ENDIAN_
	     SET_DATA_BYTE(line, 2*col, byteline[linesize-2-2*row+1]);
	     SET_DATA_BYTE(line, 2*col+1, byteline[linesize-2-2*row]);
#else  /* _BIG_ENDIAN_ */
	     SET_DATA_BYTE(line, 2*col, byteline[linesize-2-2*row]);
	     SET_DATA_BYTE(line, 2*col+1, byteline[linesize-2-2*row+1]);
#endif
	   }
	 }
	 break;
	 	   
       case 2:
	 for (row = 0; row < imagelength; row++) {
	   TIFFReadScanline(tif, buf, row, 0);
	   myline = buf;
	   line = data + (imagelength-1-row) * wpl;
	   byteline = buf;
	   for (col = 0; col < imagewidth; col++) {
#ifdef _LITTLE_ENDIAN_
	     SET_DATA_BYTE(line, linesize-2 -2*col, byteline[2*col+1]);
	     SET_DATA_BYTE(line, linesize-2 -2*col+1, byteline[2*col]);
#else  /* _BIG_ENDIAN_ */
	     SET_DATA_BYTE(line, linesize-2 -2*col, byteline[2*col]);
	     SET_DATA_BYTE(line, linesize-2 -2*col+1, byteline[2*col+1]);
#endif
	   }
	 }
	 
	 break;

	 case 3:
	 
	 for (col = 0; col < imagelength; col++) {
	   TIFFReadScanline(tif, buf, imagelength - 1 - col, 0);
	   myline = buf;
	   byteline = buf;
	   
	   for (row = 0; row < imagewidth; row++) {
	     line = data + row * wpl;
#ifdef _LITTLE_ENDIAN_
	     SET_DATA_BYTE(line, 2*col, byteline[2*row+1]);
	     SET_DATA_BYTE(line, 2*col+1, byteline[2*row]);
#else  /* _BIG_ENDIAN_ */
	     SET_DATA_BYTE(line, 2*col, byteline[l2*row]);
	     SET_DATA_BYTE(line, 2*col+1, byteline[2*row+1]);
#endif
	   }
	 }
	 break;
	 
       default:
	 fprintf (stderr, "well_edge must be one of {0,1,2,3}, %i entered\n", well_edge);
	 exit(1);
       }

     _TIFFfree(buf); 
     TIFFClose(tif);
  } else {
    pixIN = NULL;
  }
  
    return pixIN;
}

