#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>


#include "traces16.h"

#define SWAP_USHORT(Var) Var = *(unsigned short*)SwapEndian((void*)&Var, sizeof(short))

static void *SwapEndian(void* Addr, const int Nb) {
	static char Swapped[2];
	Swapped[0]=*((char*)Addr+1);
	Swapped[1]=*((char*)Addr  );
	return (void*)Swapped;
}


static PIX *pixBandLeader (PIX *pixGelRaw, struct GelInfo *GelInfo) {

PIX	*pixForBandLeader;
UINT16  max16, min16, val;
INT32   i, j;
UINT32  word;
UINT32  *lines;
UINT32  *datas;
INT32   wpls;
FLOAT32	factor;

  pixForBandLeader = pixCopy(NULL, pixGelRaw);
  datas = pixGetData(pixForBandLeader);
  wpls = pixGetWpl(pixForBandLeader);
  max16 = 0;
  min16 = 0xffff;
  for (i = 0; i < GelInfo->ImageHeight; i++) {
    lines = datas + i * wpls;
    for (j = 0; j < wpls; j++) {
      word = *(lines + j);
      max16 = MAX(max16, word >> 16);
      max16 = MAX(max16, word & 0xffff);
      min16 = MIN(min16, word >> 16);
      min16 = MIN(min16, word & 0xffff);
    }
  }
  factor =  (FLOAT32)22924. / (FLOAT32)(max16-min16);
  for (i = 0; i < GelInfo->ImageHeight; i++) {
    lines = datas + i * wpls;
    for (j = 0; j < GelInfo->ImageWidth; j++) {
      val =  (UINT16)(factor*(FLOAT32)(GET_DATA_TWO_BYTES(lines, j) - min16));
      SET_DATA_TWO_BYTES(lines, j, val);
    }
  }
  return pixForBandLeader;
}

void writeTraces16(PIX *pixGelRaw, char *tracespath, float **PeakPosition, float *NodePosition, struct GelInfo *GelInfo, int Verbose) {

char		fullName[FILE_NAME_LEN];
FILE		*TRACES16;
int		laneID, nodeID;
int		xindex, xstop, ylow, yhigh, yindex;
unsigned short 	*trace;
float		y, slope;
int		halfwidth, output_width, bufferID, buffersize;
PIX		*pixForBandLeader;
UINT32  	*linelow, *linehigh, *datas;
INT32   	wpls;


  sprintf(fullName,"%s%s",GelInfo->ProjectDir, tracespath);
  if ((TRACES16=fopen(fullName,"wb"))==NULL) {
    fprintf(stderr,"Cannot open traces16 file for writing\n");
    exit(1);
  }

  pixForBandLeader = pixBandLeader(pixGelRaw, GelInfo);
  datas = pixGetData(pixForBandLeader);
  wpls = pixGetWpl(pixForBandLeader);
  
  halfwidth = ((int)(GelInfo->LaneSpacing + 0.5)) / 2;
  output_width = 2 * halfwidth + 1;
  buffersize = output_width * GelInfo->ImageWidth;
  trace = (unsigned short *)malloc(buffersize * sizeof(unsigned short));
  fprintf(TRACES16,"TraceSize:\t%u\nTracePerLane:\t%u\nBytesPerSample:\t%u\n@@@\n",GelInfo->ImageWidth,output_width,sizeof(unsigned short));
  for (laneID = GelInfo->NumberOfLanes-1; laneID >= 0; laneID--) {
    bufferID = 0;
    for (yindex = halfwidth; yindex >= -halfwidth; yindex--) {
      for (nodeID = 0; nodeID < GelInfo->NumberOfZones+1; nodeID++) {
	slope = (PeakPosition[nodeID+1][laneID]-PeakPosition[nodeID][laneID])/(NodePosition[nodeID+1]-NodePosition[nodeID]);
	xindex = (int)(NodePosition[nodeID]+0.5);
	xstop = (int)(NodePosition[nodeID+1]+0.5);
	if (nodeID == GelInfo->NumberOfZones) xstop++;
	while (xindex < xstop) {
	  y = slope * ((float)xindex-NodePosition[nodeID])+PeakPosition[nodeID][laneID]-0.5;
	  ylow=floor(y);
	  yhigh=ceil(y);
	  linelow = datas + (ylow+yindex) * wpls;
	  if (ylow == yhigh) {
	    trace[bufferID] = GET_DATA_TWO_BYTES(linelow, xindex);
	  } else {
	    linehigh = datas + (yhigh+yindex) * wpls;
	    trace[bufferID] = floor((float)GET_DATA_TWO_BYTES(linelow, xindex)*((float)yhigh-y) +
			  (float)GET_DATA_TWO_BYTES(linehigh, xindex)*(y-(float)ylow));
	  }
	  SWAP_USHORT(trace[bufferID]);
	  bufferID++;
	  xindex++;
	}
      }
    }
    fwrite(trace,sizeof(unsigned short), buffersize,TRACES16);    
  }  

  fclose(TRACES16);
  free(trace);
  pixDestroy(&pixForBandLeader);
}
